#!/usr/bin/env python

from __future__ import print_function
from zebracorn import *
from zebracorn.x86_const import *

import regress

X86_CODE64 = "\x90" # NOP


class WriteBeforeMap(regress.RegressTest):
    def runTest(self):
        # Initialize emulator in X86-32bit mode
        mu = Uc(UC_ARCH_X86, UC_MODE_64)

        # memory address where emulation starts
        ADDRESS = 0x1000000

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, X86_CODE64)


if __name__ == '__main__':
    regress.main()



#!/usr/bin/python
from zebracorn import *
from zebracorn.x86_const import *

import regress

ESP = 0x2000
PAGE_SIZE = 1 * 1024 * 1024

#   wait
#   fnstcw  word ptr [esp]
#   pop ecx
CODE = b'\x9B\xD9\x3C\x24\x59'

def hook_mem_write(uc, access, address, size, value, user_data):
    print("mem WRITE: 0x%x, data size = %u, data value = 0x%x" % (address, size, value))
    return True

class FpuWrite(regress.RegressTest):

    def mem_reader(self, mu, addr, size, expected):
        tmp = mu.mem_read(addr, size)
        for i, e in zip(tmp, expected):
            self.assertEquals(e, i)

    def runTest(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_32)
        mu.mem_map(0, PAGE_SIZE)
        mu.mem_write(0, CODE)
        mu.reg_write(UC_X86_REG_ESP, ESP)

        mu.hook_add(UC_HOOK_MEM_WRITE, hook_mem_write)
        mu.emu_start(0x0, 5, 0, 2)
        esp = mu.reg_read(UC_X86_REG_ESP)
        self.mem_reader(mu, esp, 10, [0] * 10)

if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python
# Moshe Kravchik

from __future__ import print_function
from zebracorn import *
from zebracorn.arm_const import *
import binascii
import regress


# code to be emulated

#enable VFP
'''
00000016	f44f0370	mov.w	r3, #0xf00000
0000001a	ee013f50	mcr	p15, #0x0, r3, c1, c0, #0x2
0000bfb6	f3bf8f6f	isb	sy
0000bfba	f04f4380	mov.w	r3, #0x40000000
0000bfbe	eee83a10	vmsr	fpexc, r3
'''
ENABLE_VFP_CODE = "\x4f\xf4\x70\x03\x01\xee\x50\x3f\xbf\xf3\x6f\x8f\x4f\xf0\x80\x43\xe8\xee\x10\x3a"
VLD_CODE = "\x21\xf9\x0f\x6a"
#0000002a	f9216a0f	vld1.8	{d6, d7}, [r1]
VST_CODE = "\x00\xf9\x0f\x6a"
#0000002e	f9006a0f	vst1.8	{d6, d7}, [r0]

# memory address where emulation starts
ADDRESS    = 0x10000
SCRATCH_ADDRESS    = 0x1000

class SIMDNotReadArm(regress.RegressTest):
    def runTest(self):
        code = ENABLE_VFP_CODE+VLD_CODE+VST_CODE
        print("Emulate THUMB code")
        try:
            # Initialize emulator in thumb mode
            mu = Uc(UC_ARCH_ARM, UC_MODE_THUMB)

            # map 2MB memory for this emulation
            mu.mem_map(ADDRESS, 2 * 1024 * 1024)

            # write machine code to be emulated to memory
            mu.mem_write(ADDRESS, code)

            # map 10K scratch memory for this emulation
            mu.mem_map(SCRATCH_ADDRESS, 10 * 1024)

            # write dummy data to be emulated to memory
            mu.mem_write(SCRATCH_ADDRESS, "\x01"*64)

            # initialize machine registers
            for i in range(UC_ARM_REG_R0, UC_ARM_REG_R12):
                val = mu.reg_write(i, i - UC_ARM_REG_R0)

            mu.reg_write(UC_ARM_REG_R1, SCRATCH_ADDRESS)
            mu.reg_write(UC_ARM_REG_R0, SCRATCH_ADDRESS + 0x100)

            mu.reg_write(UC_ARM_REG_SP, 0x1234)
            mu.reg_write(UC_ARM_REG_D6, UC_ARM_REG_D6)
            mu.reg_write(UC_ARM_REG_D7, UC_ARM_REG_D7)

            print(">>> Before emulation ")
            print("\tD6 = 0x%x" % mu.reg_read(UC_ARM_REG_D6))
            print("\tD7 = 0x%x" % mu.reg_read(UC_ARM_REG_D7))
            for i in range(UC_ARM_REG_R0, UC_ARM_REG_R12):
                val = mu.reg_read(i)
                print("\t %s = 0x%x" % ("R" + str(i-UC_ARM_REG_R0),val))

            self.assertEqual(UC_ARM_REG_D6, mu.reg_read(UC_ARM_REG_D6))
            self.assertEqual(UC_ARM_REG_D7, mu.reg_read(UC_ARM_REG_D7))

            try:
                content = mu.mem_read(SCRATCH_ADDRESS, 100)
                print("Memory at addr 0x%X %s" % (SCRATCH_ADDRESS, binascii.hexlify(content)))
                content = mu.mem_read(SCRATCH_ADDRESS+0x100, 100)
                print("Memory at addr 0x%X %s" % (SCRATCH_ADDRESS+0x100, binascii.hexlify(content)))
            except Exception, errtxt:
                print (errtxt)


            # emulate machine code in infinite time
            mu.emu_start(ADDRESS, ADDRESS + len(code))

            # now print out some registers
            print(">>> Emulation done. Below is the CPU context")

            sp = mu.reg_read(UC_ARM_REG_SP)
            print(">>> SP = 0x%x" %sp)
            val = mu.reg_read(UC_ARM_REG_PC)
            print(">>> PC = 0x%x" %val)
            for i in range(UC_ARM_REG_R0, UC_ARM_REG_R12):
                val = mu.reg_read(i)
                print(">>> %s = 0x%x" % ("R" + str(i-UC_ARM_REG_R0),val))

            print("\tD6 = 0x%x" % mu.reg_read(UC_ARM_REG_D6))
            print("\tD7 = 0x%x" % mu.reg_read(UC_ARM_REG_D7))

            try:
                content = mu.mem_read(SCRATCH_ADDRESS, 100)
                print("Memory at addr 0x%X %s" % (SCRATCH_ADDRESS, binascii.hexlify(content)))
                content = mu.mem_read(SCRATCH_ADDRESS+0x100, 100)
                print("Memory at addr 0x%X %s" % (SCRATCH_ADDRESS+0x100, binascii.hexlify(content)))
            except Exception, errtxt:
                print (errtxt)

            self.assertEqual(mu.reg_read(UC_ARM_REG_D6), 0x0101010101010101)
            self.assertEqual(mu.reg_read(UC_ARM_REG_D7), 0x0101010101010101)

        except UcError as e:
            print("ERROR: %s" % e)

if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python

"""See https://github.com/zebracorn-engine/zebracorn/issues/82"""

import zebracorn
from zebracorn import *
import regress

CODE_ADDR = 0x10101000
CODE = b'\xff\xe3'  # jmp ebx

class JumEbxHang(regress.RegressTest):

    def runTest(self):
        mu = zebracorn.Uc(UC_ARCH_X86, UC_MODE_32)
        mu.mem_map(CODE_ADDR, 1024 * 4)
        mu.mem_write(CODE_ADDR, CODE)
        # If EBX is zero then an exception is raised, as expected
        mu.reg_write(zebracorn.x86_const.UC_X86_REG_EBX, 0x0)

        print(">>> jmp ebx (ebx = 0)");
        with self.assertRaises(UcError) as m:
            mu.emu_start(CODE_ADDR, CODE_ADDR + 2, count=1)

        self.assertEqual(m.exception.errno, UC_ERR_FETCH_UNMAPPED)

        print(">>> jmp ebx (ebx = 0xaa96a47f)");
        mu = zebracorn.Uc(UC_ARCH_X86, UC_MODE_32)
        mu.mem_map(CODE_ADDR, 1024 * 4)
        # If we write this address to EBX then the emulator hangs on emu_start
        mu.reg_write(zebracorn.x86_const.UC_X86_REG_EBX, 0xaa96a47f)
        mu.mem_write(CODE_ADDR, CODE)
        with self.assertRaises(UcError) as m:
            mu.emu_start(CODE_ADDR, CODE_ADDR + 2, count=1)

        self.assertEqual(m.exception.errno, UC_ERR_FETCH_UNMAPPED)

if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python

import platform
import resource

from zebracorn import *

import regress

# OS X: OK with 2047 iterations.
# OS X: Crashes at 2048:th iteration ("qemu: qemu_thread_create: Resource temporarily unavailable").
# Linux: No crashes observed.
class ThreadCreateCrash(regress.RegressTest):
    def test(self):
        for i in xrange(2048):
            Uc(UC_ARCH_X86, UC_MODE_64)
        self.assertTrue(True, "If not reached, then we have a crashing bug.")

if __name__ == '__main__':
    regress.main()



#!/usr/bin/python

from zebracorn import *
from zebracorn.x86_const import *

import regress


class Hang(regress.RegressTest):

    def runTest(self):
        PAGE_SIZE = 0x5000
        CODE_ADDR = 0x400000
        RSP_ADDR =  0x200000
        binary1 = "\xCA\x24\x5D" # retf 0x5d24
        mu = Uc(UC_ARCH_X86, UC_MODE_64)

        mu.mem_map(CODE_ADDR, PAGE_SIZE)
        mu.mem_map(RSP_ADDR, PAGE_SIZE)

        mu.mem_write(CODE_ADDR, binary1)
        mu.reg_write(UC_X86_REG_RSP, RSP_ADDR)
        try:
            self.assertEqual(mu.emu_start(CODE_ADDR, CODE_ADDR + PAGE_SIZE, 0), UC_ERR_FETCH_INVALID)
        except UcError as e:
            print("ERROR: %s" % e)


if __name__ == '__main__':
    regress.main()



#!/usr/bin/python
import regress
import zebracorn as U

class WrongEFLAGS2(regress.RegressTest):
    def test_eflags(self):
        # imul eax, ebx
        CODE = '\x0f\xaf\xc3'

        uc = U.Uc(U.UC_ARCH_X86, U.UC_MODE_32)
        uc.reg_write(U.x86_const.UC_X86_REG_EAX, 16)
        uc.reg_write(U.x86_const.UC_X86_REG_EBX, 1)
        uc.reg_write(U.x86_const.UC_X86_REG_EFLAGS, 0x292)

        uc.mem_map(0x600000, 0x1000)
        uc.mem_write(0x6000b0, CODE)
        uc.emu_start(0x6000b0, 0, count=1)


        # Here's the original execution trace for this on actual hardware.
        #
        # (gdb) x/i $eip
        # => 0x804aae5:   imul   eax,DWORD PTR [ebp-0x8]
        # (gdb) p/x $eax
        # $2 = 0x10
        # (gdb) x/wx $ebp-8
        # 0xbaaaad4c:     0x00000001
        # (gdb) p/x $eflags
        # $3 = 0x292
        # (gdb) si
        # 0x0804aae9 in ?? ()
        # (gdb) p/x $eflags
        # $4 = 0x202

        self.assertEqual(0x202, uc.reg_read(U.x86_const.UC_X86_REG_EFLAGS))

if __name__ == '__main__':
    regress.main()



#!/usr/bin/python

from zebracorn import *
from zebracorn.arm_const import *

import regress

class MovHang(regress.RegressTest):

    def runTest(self):
        uc = Uc(UC_ARCH_ARM, UC_MODE_ARM)
        uc.mem_map(0x1000, 0x1000)
        uc.mem_write(0x1000, '00c000e3'.decode('hex'))  # movw r12, #0

        def hook_block(uc, addr, *args):
            print 'enter block 0x%04x' % addr
            uc.count += 1

        uc.reg_write(UC_ARM_REG_R12, 0x123)
        self.assertEquals(uc.reg_read(UC_ARM_REG_R12), 0x123)

        uc.hook_add(UC_HOOK_BLOCK, hook_block)
        uc.count = 0

        #print 'block should only run once'
        uc.emu_start(0x1000, 0x1004, timeout=500)

        self.assertEquals(uc.reg_read(UC_ARM_REG_R12), 0x0)
        self.assertEquals(uc.count, 1)

if __name__ == '__main__':
    regress.main()



#!/usr/bin/python
from zebracorn import *
from zebracorn.mips_const import *

import regress

def hook_intr(uc, intno, _):
    print 'interrupt', intno

CODE = 0x400000
asm = '0000a48f'.decode('hex')  # lw    $a0, ($sp)

class MipsExcept(regress.RegressTest):

    def runTest(self):
        uc = Uc(UC_ARCH_MIPS, UC_MODE_MIPS32 + UC_MODE_LITTLE_ENDIAN)
        uc.hook_add(UC_HOOK_INTR, hook_intr)
        uc.mem_map(CODE, 0x1000)
        uc.mem_write(CODE, asm)

        with self.assertRaises(UcError) as m:
            uc.reg_write(UC_MIPS_REG_SP, 0x400001)
            uc.emu_start(CODE, CODE + len(asm), 300)

        self.assertEqual(UC_ERR_READ_UNALIGNED, m.exception.errno)

        with self.assertRaises(UcError) as m:
            uc.reg_write(UC_MIPS_REG_SP, 0xFFFFFFF0)
            uc.emu_start(CODE, CODE + len(asm), 200)

        self.assertEqual(UC_ERR_READ_UNMAPPED, m.exception.errno)

        with self.assertRaises(UcError) as m:
            uc.reg_write(UC_MIPS_REG_SP, 0x80000000)
            uc.emu_start(CODE, CODE + len(asm), 100)

        self.assertEqual(UC_ERR_READ_UNMAPPED, m.exception.errno)

if __name__ == '__main__':
    regress.main()




#!/usr/bin/python
# By Ryan Hileman, issue #16

from zebracorn import *
from zebracorn.arm_const import *
from zebracorn.arm64_const import *

import regress

class WrongSPArm(regress.RegressTest):

    def test_32(self):
        with self.assertRaises(UcError):
            uc = Uc(UC_ARCH_ARM, UC_MODE_32)
            uc.reg_write(UC_ARM_REG_SP, 4)

    def test_64(self):
        uc = Uc(UC_ARCH_ARM64, UC_MODE_ARM)
        uc.reg_write(UC_ARM64_REG_SP, 4)
        self.assertEqual(0x4, uc.reg_read(UC_ARM64_REG_SP))

    def test_arm(self):
        uc = Uc(UC_ARCH_ARM, UC_MODE_ARM)
        uc.reg_write(UC_ARM_REG_SP, 4)
        self.assertEqual(0x4, uc.reg_read(UC_ARM_REG_SP))

if __name__ == '__main__':
    regress.main()



#!/usr/bin/python

from zebracorn import *
from zebracorn.x86_const import *

import regress

binary1 = b'\xb8\x02\x00\x00\x00'    # mov eax, 2
binary2 = b'\xb8\x01\x00\x00\x00'    # mov eax, 1

class WrongRIP(regress.RegressTest):

    def test_step(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_64)
        mu.mem_map(0, 2 * 1024 * 1024)
        # write machine code to be emulated to memory
        mu.mem_write(0, binary1 + binary2)
        # emu for maximum 1 instruction.
        mu.emu_start(0, 5, 0, 1)

        self.assertEqual(0x2, mu.reg_read(UC_X86_REG_RAX))
        self.assertEqual(0x5, mu.reg_read(UC_X86_REG_RIP))

        mu.emu_start(5, 10, 0, 1)
        self.assertEqual(0xa, mu.reg_read(UC_X86_REG_RIP))
        self.assertEqual(0x1, mu.reg_read(UC_X86_REG_RAX))

    def test_step2(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_64)
        mu.mem_map(0, 2 * 1024 * 1024)
        # write machine code to be emulated to memory
        mu.mem_write(0, binary1 + binary2)
        # emu for maximum 1 instruction.
        mu.emu_start(0, 10, 0, 1)
        self.assertEqual(0x2, mu.reg_read(UC_X86_REG_RAX))
        self.assertEqual(0x5, mu.reg_read(UC_X86_REG_RIP))

        mu.emu_start(5, 10, 0, 1)
        self.assertEqual(0x1, mu.reg_read(UC_X86_REG_RAX))
        self.assertEqual(0xa, mu.reg_read(UC_X86_REG_RIP))

    def test_step3(self):
        bin3 = b'\x40\x01\xc1\x31\xf6' # inc eax; add ecx, eax; xor esi, esi
        mu = Uc(UC_ARCH_X86, UC_MODE_32)
        mu.mem_map(0, 2 * 1024 * 1024)
        # write machine code to be emulated to memory
        mu.mem_write(0, bin3)
        # emu for maximum 1 instruction.
        mu.emu_start(0, 10, 0, 1)
        self.assertEqual(0x1, mu.reg_read(UC_X86_REG_EAX))
        self.assertEqual(0x1, mu.reg_read(UC_X86_REG_EIP))

    def test_step_then_fin(self):
        bin4 = b'\x40\x01\xc1\x31\xf6\x90\x90\x90' # inc eax; add ecx, eax; xor esi, esi
        mu = Uc(UC_ARCH_X86, UC_MODE_32)
        mu.mem_map(0, 2 * 1024 * 1024)
        # write machine code to be emulated to memory
        mu.mem_write(0, bin4)
        # emu for maximum 1 instruction.
        mu.emu_start(0, len(binary1), 0, 1)

        self.assertEqual(0x1, mu.reg_read(UC_X86_REG_EAX))
        self.assertEqual(0x1, mu.reg_read(UC_X86_REG_EIP))
        # emu to the end
        mu.emu_start(1, len(bin4))
        self.assertEqual(0x1, mu.reg_read(UC_X86_REG_EAX))
        self.assertEqual(len(bin4), mu.reg_read(UC_X86_REG_EIP))

if __name__ == '__main__':
    regress.main()




#!/usr/bin/python

import unittest

from os.path import dirname, basename, isfile
import glob

# Find all unittest type in this directory and run it.

class RegressTest(unittest.TestCase):
    pass

def main():
    unittest.main()

if __name__ == '__main__':
    directory = dirname(__file__)
    if directory == '':
        directory = '.'
    modules = glob.glob(directory+"/*.py")
    __all__ = [ basename(f)[:-3] for f in modules if isfile(f)]
    suite = unittest.TestSuite()

    for module in __all__:
        m = __import__(module)
        for cl in dir(m):
            try:
                realcl = getattr(m,cl)
                if issubclass(realcl, unittest.TestCase):
                    suite.addTest(realcl())
            except Exception as e:
                pass

    unittest.TextTestRunner().run(suite)



#!/usr/bin/python
#
# This test demonstrates emulation behavior within and across
#  basic blocks.

from __future__ import print_function
import binascii
import regress
import struct

from zebracorn import *
from zebracorn.x86_const import *


CODE = binascii.unhexlify(b"".join([
  b"b800000000", # 1000: b8 00 00 00 00          mov    eax,0x0
  b"40",         # 1005: 40                      inc    eax
  b"40",         # 1006: 40                      inc    eax
  b"6810100000", # 1007: 68 10 10 00 00          push   0x1010
  b"c3",         # 100c: c3                      ret
  b"cc",         # 100d: cc                      int3
  b"cc",         # 100e: cc                      int3
  b"cc",         # 100f: cc                      int3
  b"b800000000", # 1010: b8 00 00 00 00          mov    eax,0x0
  b"40",         # 1015: 40                      inc    eax
  b"40",         # 1016: 40                      inc    eax 
  ]))


def showpc(mu):
    pc = mu.reg_read(UC_X86_REG_EIP)
    print("pc: 0x%x" % (pc))


class RunAcrossBBTest(regress.RegressTest):
    def test_run_all(self):
        try:
            #######################################################################
            # emu SETUP
            #######################################################################
            print("\n---- test: run_all ----")


            mu = Uc(UC_ARCH_X86, UC_MODE_32)

            def hook_code(uc, address, size, user_data):
                print(">>> Tracing instruction at 0x%x, instruction size = %u" %(address, size))
            mu.hook_add(UC_HOOK_CODE, hook_code)

            # base of CODE
            mu.mem_map(0x1000, 0x1000)
            mu.mem_write(0x1000, CODE)

            # stack
            mu.mem_map(0x2000, 0x1000)

            mu.reg_write(UC_X86_REG_EIP, 0x1000)
            mu.reg_write(UC_X86_REG_ESP, 0x2800)
            self.assertEqual(0x1000, mu.reg_read(UC_X86_REG_EIP), "unexpected PC")
            self.assertEqual(0x2800, mu.reg_read(UC_X86_REG_ESP), "unexpected SP")
            showpc(mu)

            mu.emu_start(0x1000, 0x1016)
            # should exec the following four instructions:
            # 1000: b8 00 00 00 00          mov    eax,0x0  <
            # 1005: 40                      inc    eax      <
            # 1006: 40                      inc    eax      <
            # 1007: 68 10 10 00 00          push   0x1010   <
            # 100c: c3                      ret   -----------+
            # 100d: cc                      int3             |
            # 100e: cc                      int3             |
            # 100f: cc                      int3             |
            # 1010: b8 00 00 00 00          mov    eax,0x0 <-+
            # 1015: 40                      inc    eax       <
            # 1016: 40                      inc    eax       <
            self.assertEqual(0x1016, mu.reg_read(UC_X86_REG_EIP), "unexpected PC (2)")
            self.assertEqual(0x2800, mu.reg_read(UC_X86_REG_ESP), "unexpected SP (2)")
            showpc(mu)

        except UcError as e:
            if e.errno == UC_ERR_FETCH_UNMAPPED:
                # during initial test dev, bad fetch at 0x1010, but the data is there,
                #   and this proves it
                print("!!! about to bail due to bad fetch... here's the data at PC:")
                print(binascii.hexlify(mu.mem_read(mu.reg_read(UC_X86_REG_EIP), 0x8)))

            self.assertFalse(True, "ERROR: %s @ 0x%x" % (e, mu.reg_read(UC_X86_REG_EIP)))



    def test_run_across_bb(self):
        try:
            #######################################################################
            # emu SETUP
            #######################################################################
            print("\n---- test: run_across_bb ----")


            mu = Uc(UC_ARCH_X86, UC_MODE_32)

            def hook_code(uc, address, size, user_data):
                print(">>> Tracing instruction at 0x%x, instruction size = %u" %(address, size))
            mu.hook_add(UC_HOOK_CODE, hook_code)

            # base of CODE
            mu.mem_map(0x1000, 0x1000)
            mu.mem_write(0x1000, CODE)

            # stack
            mu.mem_map(0x2000, 0x1000)

            mu.reg_write(UC_X86_REG_EIP, 0x1000)
            mu.reg_write(UC_X86_REG_ESP, 0x2800)
            self.assertEqual(0x1000, mu.reg_read(UC_X86_REG_EIP), "unexpected PC")
            self.assertEqual(0x2800, mu.reg_read(UC_X86_REG_ESP), "unexpected SP")
            showpc(mu)


            #######################################################################
            # emu_run ONE:
            #   exectue four instructions, until the last instruction in a BB
            #######################################################################


            mu.emu_start(0x1000, 0x100c)
            # should exec the following four instructions:
            # 1000: b8 00 00 00 00          mov    eax,0x0  <
            # 1005: 40                      inc    eax      <
            # 1006: 40                      inc    eax      <
            # 1007: 68 10 10 00 00          push   0x1010   <

            # should be at 0x100c, as requested
            self.assertEqual(0x100c, mu.reg_read(UC_X86_REG_EIP), "unexpected PC (2)")

            # single push, so stack diff is 0x4
            TOP_OF_STACK = 0x2800-0x4
            self.assertEqual(TOP_OF_STACK, mu.reg_read(UC_X86_REG_ESP), "unexpected SP (2)")

            # top of stack should be 0x1010
            self.assertEqual(0x1010, 
                             struct.unpack("<I", mu.mem_read(TOP_OF_STACK, 0x4))[0],
                             "unexpected stack value")
            showpc(mu)


            #######################################################################
            # emu_run TWO
            #   execute one instruction that jumps to a new BB
            #######################################################################


            mu.emu_start(0x100c, 0x1010)
            # should exec one instruction that jumps to 0x1010:
            # 100c: c3                      ret   -----------+
            # 100d: cc                      int3             |
            # 100e: cc                      int3             |
            # 100f: cc                      int3             |
            # 1010: b8 00 00 00 00          mov    eax,0x0 <-+

            # should be at 0x1010, as requested
            self.assertEqual(0x1010, mu.reg_read(UC_X86_REG_EIP), "unexpected PC (3)")

            # single pop, so stack back at base
            self.assertEqual(0x2800, mu.reg_read(UC_X86_REG_ESP), "unexpected SP (3)")
            showpc(mu)


            #######################################################################
            # emu_run THREE
            #  execute three instructions to verify things work as expected
            #######################################################################


            mu.emu_start(0x1010, 0x1016)
            # should exec the following three instructions:
            # 1010: b8 00 00 00 00          mov    eax,0x0   <
            # 1015: 40                      inc    eax       <
            # 1016: 40                      inc    eax       <
            self.assertEqual(0x1016, mu.reg_read(UC_X86_REG_EIP),
	                     "unexpected PC (4): 0x%x vs 0x%x" % (
				     0x1016, mu.reg_read(UC_X86_REG_EIP)))
            showpc(mu)

        except UcError as e:
            if e.errno == UC_ERR_FETCH_UNMAPPED:
                # during initial test dev, bad fetch at 0x1010, but the data is there,
                #   and this proves it
                print("!!! about to bail due to bad fetch... here's the data at PC:")
                print(binascii.hexlify(mu.mem_read(mu.reg_read(UC_X86_REG_EIP), 0x8)))

            self.assertFalse(True, "ERROR: %s @ 0x%x" % (e, mu.reg_read(UC_X86_REG_EIP)))


if __name__ == '__main__':
    regress.main()



#!/usr/bin/python

'''https://github.com/zebracorn-engine/zebracorn/issues/334'''

from __future__ import print_function
import regress

from zebracorn import *
from zebracorn.x86_const import *

ADDRESS = 0x8048000
STACK_ADDRESS = 0xffff000
STACK_SIZE = 4096
'''
31 DB           xor ebx, ebx
53              push ebx
43              inc ebx
53              push ebx
6A 02           push 2
6A 66           push 66h
58              pop eax
89 E1           mov ecx, esp
CD 80           int 80h
'''
CODE = "\x31\xDB\x53\x43\x53\x6A\x02\x6A\x66\x58\x89\xE1\xCD\x80"
EP = ADDRESS + 0x54

def hook_code(mu, address, size, user_data):
        print(">>> Tracing instruction at 0x%x, instruction size = %u" %(address, size))
        
class HookCodeAddDelTest(regress.RegressTest):
        def runTest(self):
                emu = Uc(UC_ARCH_X86, UC_MODE_32)
                emu.mem_map(ADDRESS, 0x1000)
                emu.mem_write(EP, CODE)

                emu.mem_map(STACK_ADDRESS, STACK_SIZE)
                emu.reg_write(UC_X86_REG_ESP, STACK_ADDRESS + STACK_SIZE)
                
                # UC_HOOK_CODE hook will work even after deletion
                i = emu.hook_add(UC_HOOK_CODE, hook_code, None)
                emu.hook_del(i)
                
                emu.emu_start(EP, EP + len(CODE), count = 3)
                print("EIP: 0x%x" % emu.reg_read(UC_X86_REG_EIP))
                
if __name__ == '__main__':
        regress.main()



#!/usr/bin/env python
# Mariano Graziano

from zebracorn import *
from zebracorn.x86_const import *

import regress

#echo -ne "\x48\x31\xc0\x48\xb8\x04\x00\x00\x00\x00\x00\x00\x00\x48\x3d\x05\x00\x00\x00\x74\x05\xe9\x0f\x00\x00\x00\x48\xba\xbe\xba\x00\x00\x00\x00\x00\x00\xe9\x0f\x00\x00\x00\x48\xba\xca\xc0\x00\x00\x00\x00\x00\x00\xe9\x00\x00\x00\x00\x90" | ndisasm - -b64
#00000000  4831C0            xor rax,rax
#00000003  48B8040000000000  mov rax,0x4
#         -0000
#0000000D  483D05000000      cmp rax,0x5
#00000013  7405              jz 0x1a
#00000015  E90F000000        jmp qword 0x29
#0000001A  48BABEBA00000000  mov rdx,0xbabe
#         -0000
#00000024  E90F000000        jmp qword 0x38
#00000029  48BACAC000000000  mov rdx,0xc0ca
#         -0000
#00000033  E900000000        jmp qword 0x38
#00000038  90                nop


mu = 0
zf = 1 # (0:clear, 1:set)


class Init(regress.RegressTest):
    def clear_zf(self):
        eflags_cur = mu.reg_read(UC_X86_REG_EFLAGS)
        eflags = eflags_cur & ~(1 << 6)
        #eflags = 0x0
        print "[clear_zf] - eflags from %x to %x" % (eflags_cur, eflags)
        if eflags != eflags_cur:
            print "[clear_zf] - writing new eflags..."
            mu.reg_write(UC_X86_REG_EFLAGS, eflags)

    def set_zf(self):
        eflags_cur = mu.reg_read(UC_X86_REG_EFLAGS)
        eflags = eflags_cur | (1 << 6)
        #eflags = 0xFFFFFFFF
        print "[set_zf] - eflags from %x to %x" % (eflags_cur, eflags)
        if eflags != eflags_cur:
            print "[set_zf] - writing new eflags..."
            mu.reg_write(UC_X86_REG_EFLAGS, eflags)

    def handle_zf(self, zf): 
        print "[handle_zf] - eflags " , zf
        if zf == 0: self.clear_zf()
        else: self.set_zf()

    def multipath(self):
        print "[multipath] - handling ZF (%s) - default" % zf
        self.handle_zf(zf)

    # callback for tracing basic blocks
    def hook_block(self, uc, address, size, user_data):
        print(">>> Tracing basic block at 0x%x, block size = 0x%x" %(address, size))

    # callback for tracing instructions
    def hook_code(self, uc, address, size, user_data):
        print(">>> Tracing instruction at 0x%x, instruction size = %u" %(address, size))
        rax = mu.reg_read(UC_X86_REG_RAX)
        rbx = mu.reg_read(UC_X86_REG_RBX)
        rcx = mu.reg_read(UC_X86_REG_RCX)
        rdx = mu.reg_read(UC_X86_REG_RDX)
        rsi = mu.reg_read(UC_X86_REG_RSI)
        rdi = mu.reg_read(UC_X86_REG_RDI)
        r8 = mu.reg_read(UC_X86_REG_R8)
        r9 = mu.reg_read(UC_X86_REG_R9)
        r10 = mu.reg_read(UC_X86_REG_R10)
        r11 = mu.reg_read(UC_X86_REG_R11)
        r12 = mu.reg_read(UC_X86_REG_R12)
        r13 = mu.reg_read(UC_X86_REG_R13)
        r14 = mu.reg_read(UC_X86_REG_R14)
        r15 = mu.reg_read(UC_X86_REG_R15)
        eflags = mu.reg_read(UC_X86_REG_EFLAGS)
        
        print(">>> RAX = %x" %rax)
        print(">>> RBX = %x" %rbx)
        print(">>> RCX = %x" %rcx)
        print(">>> RDX = %x" %rdx)
        print(">>> RSI = %x" %rsi)
        print(">>> RDI = %x" %rdi)
        print(">>> R8 = %x" %r8)
        print(">>> R9 = %x" %r9)
        print(">>> R10 = %x" %r10)
        print(">>> R11 = %x" %r11)
        print(">>> R12 = %x" %r12)
        print(">>> R13 = %x" %r13)
        print(">>> R14 = %x" %r14)
        print(">>> R15 = %x" %r15)
        print(">>> ELAGS = %x" %eflags)
        print "-"*11
        self.multipath()
        print "-"*11

    # callback for tracing memory access (READ or WRITE)
    def hook_mem_access(self, uc, access, address, size, value, user_data):
        if access == UC_MEM_WRITE:
            print(">>> Memory is being WRITE at 0x%x, data size = %u, data value = 0x%x" \
                    %(address, size, value))
        else:   # READ
            print(">>> Memory is being READ at 0x%x, data size = %u" \
                    %(address, size))

    # callback for tracing invalid memory access (READ or WRITE)
    def hook_mem_invalid(self, uc, access, address, size, value, user_data):
        print("[ HOOK_MEM_INVALID - Address: %s ]" % hex(address))
        if access == UC_MEM_WRITE_UNMAPPED:
            print(">>> Missing memory is being WRITE at 0x%x, data size = %u, data value = 0x%x" %(address, size, value))
            return True
        else:
            print(">>> Missing memory is being READ at 0x%x, data size = %u, data value = 0x%x" %(address, size, value))
            return True


    def hook_mem_fetch_unmapped(self, uc, access, address, size, value, user_data):
        print("[ HOOK_MEM_FETCH - Address: %s ]" % hex(address))
        print("[ mem_fetch_unmapped: faulting address at %s ]" % hex(address).strip("L"))
        return True

    def runTest(self):
        global mu
 
        JUMP = "\x48\x31\xc0\x48\xb8\x04\x00\x00\x00\x00\x00\x00\x00\x48\x3d\x05\x00\x00\x00\x74\x05\xe9\x0f\x00\x00\x00\x48\xba\xbe\xba\x00\x00\x00\x00\x00\x00\xe9\x0f\x00\x00\x00\x48\xba\xca\xc0\x00\x00\x00\x00\x00\x00\xe9\x00\x00\x00\x00\x90"

        ADDRESS = 0x1000000

        print("Emulate x86_64 code")
        # Initialize emulator in X86-64bit mode
        mu = Uc(UC_ARCH_X86, UC_MODE_64)

        # map 2MB memory for this emulation
        mu.mem_map(ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, JUMP)

        # setup stack
        mu.reg_write(UC_X86_REG_RSP, ADDRESS + 0x200000)

        # tracing all basic blocks with customized callback
        mu.hook_add(UC_HOOK_BLOCK, self.hook_block)

        # tracing all instructions in range [ADDRESS, ADDRESS+0x60]
        mu.hook_add(UC_HOOK_CODE, self.hook_code, None, ADDRESS, ADDRESS+0x60)

        # tracing all memory READ & WRITE access
        mu.hook_add(UC_HOOK_MEM_WRITE, self.hook_mem_access)
        mu.hook_add(UC_HOOK_MEM_READ, self.hook_mem_access)
        mu.hook_add(UC_HOOK_MEM_FETCH_UNMAPPED, self.hook_mem_fetch_unmapped)
        mu.hook_add(UC_HOOK_MEM_READ_UNMAPPED | UC_HOOK_MEM_WRITE_UNMAPPED, self.hook_mem_invalid)

        try:
            # emulate machine code in infinite time
            mu.emu_start(ADDRESS, ADDRESS + len(JUMP))
        except UcError as e:
            print("ERROR: %s" % e)

        rdx = mu.reg_read(UC_X86_REG_RDX)
        self.assertEqual(rdx, 0xbabe, "RDX contains the wrong value. Eflags modification failed.")


if __name__ == '__main__':
    regress.main()



#!/usr/bin/python
# By Ryan Hileman, issue #3

from capstone import *
from zebracorn import *
from zebracorn.x86_const import *

import regress
code = 'f20f1005aa120000'.decode('hex')

def dis(mem, addr):
    md = Cs(CS_ARCH_X86, CS_MODE_64)
    return '\n'.join([
        '%s %s' % (i.mnemonic, i.op_str)
        for i in md.disasm(str(mem), addr)
    ])

def hook_code(uc, addr, size, user_data):
    mem = uc.mem_read(addr, size)
    print 'instruction size:', size
    print 'instruction:', str(mem).encode('hex'), dis(mem, addr)
    print 'reference:  ', code.encode('hex'), dis(code, addr)

class Movsd(regress.RegressTest):

    def runTest(self):
        addr = 0x400000
        mu = Uc(UC_ARCH_X86, UC_MODE_64)
        mu.hook_add(UC_HOOK_CODE, hook_code)
        mu.mem_map(addr, 8 * 1024 * 1024)
        mu.mem_write(addr, code)
        mu.emu_start(addr, addr + len(code))

if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python
# Sample code for ARM of Unicorn. Nguyen Anh Quynh <aquynh@gmail.com>
# Python sample ported by Loi Anh Tuan <loianhtuan@gmail.com>
#


from __future__ import print_function
from zebracorn import *
from zebracorn.arm_const import *


# code to be emulated
ARM_CODE   = "\x37\x00\xa0\xe3\x03\x10\x42\xe0" # mov r0, #0x37; sub r1, r2, r3
THUMB_CODE = "\x83\xb0" # sub    sp, #0xc
# memory address where emulation starts
ADDRESS    = 0xF0000000


# callback for tracing basic blocks
def hook_block(uc, address, size, user_data):
    print(">>> Tracing basic block at 0x%x, block size = 0x%x" %(address, size))


# callback for tracing instructions
def hook_code(uc, address, size, user_data):
    print(">>> Tracing instruction at 0x%x, instruction size = %u" %(address, size))


# Test ARM
def test_arm():
    print("Emulate ARM code")
    try:
        # Initialize emulator in ARM mode
        mu = Uc(UC_ARCH_ARM, UC_MODE_ARM)
        
        mem_size = 2 * (1024 * 1024)
        mu.mem_map(ADDRESS, mem_size)
        
        stack_address = ADDRESS + mem_size
        stack_size =  stack_address         # >>> here huge memory size
        mu.mem_map(stack_address, stack_size)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, ARM_CODE)

        # initialize machine registers
        mu.reg_write(UC_ARM_REG_R0, 0x1234)
        mu.reg_write(UC_ARM_REG_R2, 0x6789)
        mu.reg_write(UC_ARM_REG_R3, 0x3333)

        # tracing all basic blocks with customized callback
        mu.hook_add(UC_HOOK_BLOCK, hook_block)

        # tracing all instructions with customized callback
        mu.hook_add(UC_HOOK_CODE, hook_code)

        # emulate machine code in infinite time
        mu.emu_start(ADDRESS, ADDRESS + len(ARM_CODE))

        # now print out some registers
        print(">>> Emulation done. Below is the CPU context")

        r0 = mu.reg_read(UC_ARM_REG_R0)
        r1 = mu.reg_read(UC_ARM_REG_R1)
        print(">>> R0 = 0x%x" %r0)
        print(">>> R1 = 0x%x" %r1)

    except UcError as e:
        print("ERROR: %s" % e)


def test_thumb():
    print("Emulate THUMB code")
    try:
        # Initialize emulator in thumb mode
        mu = Uc(UC_ARCH_ARM, UC_MODE_THUMB)

        # map 2MB memory for this emulation
        mu.mem_map(ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, THUMB_CODE)

        # initialize machine registers
        mu.reg_write(UC_ARM_REG_SP, 0x1234)

        # tracing all basic blocks with customized callback
        mu.hook_add(UC_HOOK_BLOCK, hook_block)

        # tracing all instructions with customized callback
        mu.hook_add(UC_HOOK_CODE, hook_code)

        # emulate machine code in infinite time
        mu.emu_start(ADDRESS, ADDRESS + len(THUMB_CODE))

        # now print out some registers
        print(">>> Emulation done. Below is the CPU context")

        sp = mu.reg_read(UC_ARM_REG_SP)
        print(">>> SP = 0x%x" %sp)

    except UcError as e:
        print("ERROR: %s" % e)


if __name__ == '__main__':
    test_arm()
    print("=" * 20)
    test_thumb() 



#!/usr/bin/python

from zebracorn import *
from zebracorn.x86_const import *

import regress

class VldrPcInsn(regress.RegressTest):

    def runTest(self):
        uc = Uc(UC_ARCH_X86, UC_MODE_32)
        uc.mem_map(0x1000, 0x1000)
        # mov gs, eax; mov eax, 1
        code = '8ee8b801000000'.decode('hex')
        uc.mem_write(0x1000, code)
        uc.reg_write(UC_X86_REG_EAX, 0xFFFFFFFF)

        with self.assertRaises(UcError) as ex_ctx:
            uc.emu_start(0x1000, 0x1000 + len(code))

        self.assertEquals(ex_ctx.exception.errno, UC_ERR_EXCEPTION)

if __name__ == '__main__':
    regress.main()



#!/usr/bin/python
from zebracorn import *
from zebracorn.x86_const import *
from capstone import *
import regress

ESP = 0x2000
PAGE_SIZE = 2 * 1024 * 1024

#	mov [esp], DWORD 0x37f
#	fldcw [esp]
#	fnop
#	fnstenv [esp + 8]
#	pop ecx
CODE = b'\xc7\x04\x24\x7f\x03\x00\x00\xd9\x2c\x24\xd9\xd0\xd9\x74\x24\x08\x59'

class SimpleEngine:
	def __init__(self):
		self.capmd = Cs(CS_ARCH_X86, CS_MODE_32)

	def disas_single(self, data):
		for i in self.capmd.disasm(data, 16):
			print("\t%s\t%s" % (i.mnemonic, i.op_str))
			break

disasm = SimpleEngine()

def hook_code(uc, addr, size, user_data):
	mem = uc.mem_read(addr, size)
	print("  0x%X:" % (addr)),
	disasm.disas_single(str(mem))

class FpuIP(regress.RegressTest):

    def mem_reader(self, mu, addr, size, expected):
        tmp = mu.mem_read(addr, size)
        for out, exp in zip(tmp, expected):
            self.assertEqual(exp, out)

    def test_32(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_32)

        mu.mem_map(0x0, PAGE_SIZE)
        mu.mem_write(0x4000, CODE)
        mu.reg_write(UC_X86_REG_ESP, ESP)
        mu.hook_add(UC_HOOK_CODE, hook_code)

        mu.emu_start(0x4000, 0, 0, 5)
        esp = mu.reg_read(UC_X86_REG_ESP)
        self.assertEqual(0x2004, esp)
        expected = [0x0, 0x0, 0xa, 0x40]
        self.mem_reader(mu, esp +  14, 4, expected)

    def test_64(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_64)

        mu.mem_map(0x0, PAGE_SIZE)
        mu.mem_write(0x4000, CODE)
        mu.reg_write(UC_X86_REG_ESP, ESP)
        mu.hook_add(UC_HOOK_CODE, hook_code)

        mu.emu_start(0x4000, 0, 0, 5)
        rsp = mu.reg_read(UC_X86_REG_RSP)
        self.assertEqual(0x2012, rsp + 10)
        expected = [0x0, 0x0, 0xa, 0x40, 0x0, 0x0, 0x0, 0x0]
        self.mem_reader(mu, rsp + 10, 4, expected)

if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python

import platform
import resource

from zebracorn import *

import regress

class MemoryLeak(regress.RegressTest):
    def test(self):
        if platform.system() == "Darwin":
            rusage_multiplier = 1
        elif platform.system() == "Linux":
            rusage_multiplier = 1024
        else:
            # resource.getrusage(...) is platform dependent. Only tested under OS X and Linux.
            return
        max_rss_before = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss * rusage_multiplier
        for i in xrange(10000):
            mu = Uc(UC_ARCH_X86, UC_MODE_64)
            mu.mem_map(0, 4096)
            mu.emu_start(0, 0)
        max_rss_after = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss * rusage_multiplier
        rss_increase_per_iteration = (max_rss_after - max_rss_before) / i
        self.assertLess(rss_increase_per_iteration, 8000)

if __name__ == '__main__':
    regress.main()



#!/usr/bin/python

from zebracorn import *
from zebracorn.x86_const import *

import regress

CODE_ADDR = 0x0
binary1 = b'\xb8\x02\x00\x00\x00'
binary2 = b'\xb8\x01\x00\x00\x00'

class CrashTB(regress.RegressTest):

    def runTest(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_64)

        mu.mem_map(CODE_ADDR, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(CODE_ADDR, binary1)

        # emu for maximum 1 sec.
        mu.emu_start(CODE_ADDR, len(binary1), UC_SECOND_SCALE)

        self.assertEqual(0x2, mu.reg_read(UC_X86_REG_RAX))

        # write machine code to be emulated to memory
        mu.mem_write(CODE_ADDR, binary2)

        # emu for maximum 1 sec.
        mu.emu_start(CODE_ADDR, len(binary2), UC_SECOND_SCALE)

        self.assertEqual(0x1, mu.reg_read(UC_X86_REG_RAX))

if __name__ == '__main__':
    regress.main()




#!/usr/bin/python
from zebracorn import *
from zebracorn.x86_const import *
import regress

PAGE_SIZE = 4 * 1024
ACCESS_ADDR = 0x1000

# mov eax, [0x1000]
# mov eax, [0x1000]
CODE = b'\xA1\x00\x10\x00\x00\xA1\x00\x10\x00\x00'

def hook_mem_read(uc, access, address, size, value, data):
    print("Reading at " + str(address))
    uc.mem_write(address, CODE);

class REP(regress.RegressTest):

    def test_rep(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_32)

        mu.mem_map(0, PAGE_SIZE)
        mu.mem_write(0, CODE)
        mu.mem_map(ACCESS_ADDR, PAGE_SIZE, UC_PROT_READ);
        mu.hook_add(UC_HOOK_MEM_READ, hook_mem_read, begin = ACCESS_ADDR, end = ACCESS_ADDR + PAGE_SIZE)

        mu.emu_start(0, len(CODE))

if __name__ == '__main__':
    regress.main()



from __future__ import print_function
from zebracorn import *
from zebracorn.arm_const import *
import regress

# code to be emulated
'''
ins = {
    0x00008cd4: """
                push    {r11}
                add     r11, sp, #0
                mov     r3, pc
                mov     r0, r3
                sub     sp, r11, #0
                pop     {r11}
                bx      lr
                """,
    0x00008cf0: """
                push    {r11}
                add     r11, sp, #0
                push   {r6}
                add    r6, pc, $1
                bx r6
                .code   16
                mov    r3, pc
                add    r3, $0x4
                push   {r3}
                pop    {pc}
                .code  32
                pop    {r6}
                mov     r0, r3
                sub     sp, r11, #0
                pop     {r11}
                bx      lr
                """,
    0x00008d20: """
                push    {r11}
                add r11, sp, #0
                mov r3, lr
                mov r0, r3
                sub sp, r11, #0
                pop {r11}
                bx  lr
                """,
    0x00008d68: "bl      0x8cd4\n"
                "mov     r4, r0\n"
                "bl      0x8cf0\n"
                "mov     r3, r0\n"
                "add     r4, r4, r3\n"
                "bl      0x8d20\n"
                "mov     r3, r0\n"
                "add     r2, r4, r3",
}
'''

class BxTwiceTest(regress.RegressTest):
    def runTest(self):
        ADDRESS = 0x8000
        MAIN_ADDRESS = 0x8d68
        STACK_ADDR = ADDRESS + 0x1000

        code = {
            0x8cf0: '\x04\xb0-\xe5\x00\xb0\x8d\xe2\x04`-\xe5\x01`\x8f\xe2\x16\xff/\xe1{F\x03\xf1\x04\x03\x08\xb4\x00\xbd\x00\x00\x04`\x9d\xe4\x03\x00\xa0\xe1\x00\xd0K\xe2\x04\xb0\x9d\xe4\x1e\xff/\xe1',
            0x8d20: '\x04\xb0-\xe5\x00\xb0\x8d\xe2\x0e0\xa0\xe1\x03\x00\xa0\xe1\x00\xd0K\xe2\x04\xb0\x9d\xe4\x1e\xff/\xe1',
            0x8cd4: '\x04\xb0-\xe5\x00\xb0\x8d\xe2\x0f0\xa0\xe1\x03\x00\xa0\xe1\x00\xd0K\xe2\x04\xb0\x9d\xe4\x1e\xff/\xe1',
            0x8d68: '\xd9\xff\xff\xeb\x00@\xa0\xe1\xde\xff\xff\xeb\x000\xa0\xe1\x03@\x84\xe0\xe7\xff\xff\xeb\x000\xa0\xe1\x03 \x84\xe0'
        }

        try:
            mu = Uc(UC_ARCH_ARM, UC_MODE_ARM)
            # map 2MB memory for this emulation
            mu.mem_map(ADDRESS, 2 * 1024 * 1024)

            # write machine code to be emulated to memory
            for addr, c in code.items():
                print("Writing chunk to 0x{:x}".format(addr))
                mu.mem_write(addr, c)

            # initialize machine registers
            mu.reg_write(UC_ARM_REG_SP, STACK_ADDR)

            print("Starting emulation")

            # emulate code in infinite time & unlimited instructions
            mu.emu_start(MAIN_ADDRESS, MAIN_ADDRESS + len(code[MAIN_ADDRESS]))

            print("Emulation done")

            r2 = mu.reg_read(UC_ARM_REG_R2)
            print(">>> r2: 0x{:08x}".format(r2))

        except UcError as e:
            self.fail("ERROR: %s" % e)



# Copyright (c) 2009 Raymond Hettinger
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
#     The above copyright notice and this permission notice shall be
#     included in all copies or substantial portions of the Software.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#     EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
#     OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
#     NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
#     WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#     FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
#     OTHER DEALINGS IN THE SOFTWARE.

try:
    from UserDict import UserDict
    from UserDict import DictMixin
except ImportError:
    from collections import UserDict
    try:
        from collections import MutableMapping as DictMixin
    except ImportError:
        from collections.abc import MutableMapping as DictMixin

class OrderedDict(dict, DictMixin):

    def __init__(self, *args, **kwds):
        if len(args) > 1:
            raise TypeError('expected at most 1 arguments, got %d' % len(args))
        try:
            self.__end
        except AttributeError:
            self.clear()
        self.update(*args, **kwds)

    def clear(self):
        self.__end = end = []
        end += [None, end, end]         # sentinel node for doubly linked list
        self.__map = {}                 # key --> [key, prev, next]
        dict.clear(self)

    def __setitem__(self, key, value):
        if key not in self:
            end = self.__end
            curr = end[1]
            curr[2] = end[1] = self.__map[key] = [key, curr, end]
        dict.__setitem__(self, key, value)

    def __delitem__(self, key):
        dict.__delitem__(self, key)
        key, prev, next = self.__map.pop(key)
        prev[2] = next
        next[1] = prev

    def __iter__(self):
        end = self.__end
        curr = end[2]
        while curr is not end:
            yield curr[0]
            curr = curr[2]

    def __reversed__(self):
        end = self.__end
        curr = end[1]
        while curr is not end:
            yield curr[0]
            curr = curr[1]

    def popitem(self, last=True):
        if not self:
            raise KeyError('dictionary is empty')
        if last:
            key = reversed(self).next()
        else:
            key = iter(self).next()
        value = self.pop(key)
        return key, value

    def __reduce__(self):
        items = [[k, self[k]] for k in self]
        tmp = self.__map, self.__end
        del self.__map, self.__end
        inst_dict = vars(self).copy()
        self.__map, self.__end = tmp
        if inst_dict:
            return (self.__class__, (items,), inst_dict)
        return self.__class__, (items,)

    def keys(self):
        return list(self)

    setdefault = DictMixin.setdefault
    update = DictMixin.update
    pop = DictMixin.pop
    values = DictMixin.values
    items = DictMixin.items

    def __repr__(self):
        if not self:
            return '%s()' % (self.__class__.__name__,)
        return '%s(%r)' % (self.__class__.__name__, self.items())

    def copy(self):
        return self.__class__(self)

    @classmethod
    def fromkeys(cls, iterable, value=None):
        d = cls()
        for key in iterable:
            d[key] = value
        return d

    def __eq__(self, other):
        if isinstance(other, OrderedDict):
            if len(self) != len(other):
                return False
            for p, q in  zip(self.items(), other.items()):
                if p != q:
                    return False
            return True
        return dict.__eq__(self, other)

    def __ne__(self, other):
        return not self == other



#!/usr/bin/python
# By Ryan Hileman, issue #9

# this prints out 2 lines and the contents must be the same

from zebracorn import *
import regress

class MemMap(regress.RegressTest):

    def test_mmap_write(self):
        uc = Uc(UC_ARCH_X86, UC_MODE_64)

        uc.mem_map(0x8048000, 0x2000)
        uc.mem_write(0x8048000, 'test')
        s1 = str(uc.mem_read(0x8048000, 4)).encode('hex')

        self.assertEqual('test'.encode('hex'), s1)

        uc.mem_map(0x804a000, 0x8000)
        s2 = str(uc.mem_read(0x8048000, 4)).encode('hex')
        self.assertEqual(s1, s2)

    def test_mmap_invalid(self):
        u = zebracorn.Uc(zebracorn.UC_ARCH_X86, zebracorn.UC_MODE_32)
        with self.assertRaises(UcError):
            u.mem_map(0x2000, 0)
        with self.assertRaises(UcError):
            u.mem_map(0x4000, 1)

    def test_mmap_weird(self):
        u = zebracorn.Uc(zebracorn.UC_ARCH_X86, zebracorn.UC_MODE_32)

        for i in xrange(20):
            with self.assertRaises(UcError):
                u.mem_map(i*0x1000, 5)
                u.mem_read(i*0x1000+6, 1)

if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python
from zebracorn import *
from zebracorn.x86_const import *
from struct import pack

import regress

CODE_ADDR = 0x1000000
CODE = (
    '\xb8\x00\x00\x00\x02' # mov eax, 0x2000000
    '\xdb\x28'             # fldt [eax]
    '\xd9\xfa'             # fsqrt
)

DATA_ADDR = 0x2000000
DATA = '\0\0\0\0\0\0\0\0\0\1'

class FldtFsqrt(regress.RegressTest):
    def test_fldt_fsqrt(self):
        uc = Uc(UC_ARCH_X86, UC_MODE_32)

        uc.mem_map(CODE_ADDR, 0x1000)
        uc.mem_write(CODE_ADDR, CODE)

        uc.mem_map(DATA_ADDR, 0x1000)
        uc.mem_write(DATA_ADDR, DATA)

        uc.emu_start(CODE_ADDR, CODE_ADDR + len(CODE), 10000, 10)

if __name__ == '__main__':
    regress.main()



# For Unicorn Engine. AUTO-GENERATED FILE, DO NOT EDIT [m68k_const.py]

# M68K registers

UC_M68K_REG_INVALID = 0
UC_M68K_REG_A0 = 1
UC_M68K_REG_A1 = 2
UC_M68K_REG_A2 = 3
UC_M68K_REG_A3 = 4
UC_M68K_REG_A4 = 5
UC_M68K_REG_A5 = 6
UC_M68K_REG_A6 = 7
UC_M68K_REG_A7 = 8
UC_M68K_REG_D0 = 9
UC_M68K_REG_D1 = 10
UC_M68K_REG_D2 = 11
UC_M68K_REG_D3 = 12
UC_M68K_REG_D4 = 13
UC_M68K_REG_D5 = 14
UC_M68K_REG_D6 = 15
UC_M68K_REG_D7 = 16
UC_M68K_REG_SR = 17
UC_M68K_REG_PC = 18
UC_M68K_REG_ENDING = 19



#!/usr/bin/python
import regress
import zebracorn as U

class WrongEFLAGS(regress.RegressTest):
    def test_eflags(self):
        # xor r14,r14
        CODE = 'M1\xf6'

        uc = U.Uc(U.UC_ARCH_X86, U.UC_MODE_64)
        uc.reg_write(U.x86_const.UC_X86_REG_RIP, 0x6000b0)
        uc.reg_write(U.x86_const.UC_X86_REG_EFLAGS, 0x200)

        uc.mem_map(0x600000, 0x1000)
        uc.mem_write(0x6000b0, CODE)
        uc.emu_start(0x6000b0, 0, count=1)


        # Here's the original execution trace for this on actual hardware.
        #
        # (gdb) x/i $pc
        # => 0x6000b0:    xor    %r14,%r14
        # (gdb) p/x $eflags
        # $1 = 0x200
        # (gdb) p $eflags
        # $2 = [ IF ]
        # (gdb) si
        # 0x00000000006000b3 in ?? ()
        # (gdb) p/x $eflags
        # $3 = 0x246
        # (gdb) p $eflags
        # $4 = [ PF ZF IF ]

        self.assertEqual(0x6000b3, uc.reg_read(U.x86_const.UC_X86_REG_RIP))
        self.assertEqual(0x246, uc.reg_read(U.x86_const.UC_X86_REG_EFLAGS))

if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python
# Python binding for Unicorn engine. Nguyen Anh Quynh <aquynh@gmail.com>

from __future__ import print_function
import glob
import os
import subprocess
import shutil
import sys
import platform

from distutils import log
from distutils.core import setup
from distutils.util import get_platform
from distutils.command.build import build
from distutils.command.sdist import sdist
from setuptools.command.bdist_egg import bdist_egg

SYSTEM = sys.platform

# sys.maxint is 2**31 - 1 on both 32 and 64 bit mingw
IS_64BITS = platform.architecture()[0] == '64bit'

# are we building from the repository or from a source distribution?
ROOT_DIR = os.path.dirname(os.path.realpath(__file__))
LIBS_DIR = os.path.join(ROOT_DIR, 'unicorn', 'lib')
HEADERS_DIR = os.path.join(ROOT_DIR, 'unicorn', 'include')
SRC_DIR = os.path.join(ROOT_DIR, 'src')
BUILD_DIR = SRC_DIR if os.path.exists(SRC_DIR) else os.path.join(ROOT_DIR, '../..')

# Parse version from pkgconfig.mk
VERSION_DATA = {}
with open(os.path.join(BUILD_DIR, 'pkgconfig.mk')) as fp:
    lines = fp.readlines()
    for line in lines:
        line = line.strip()
        if len(line) == 0:
            continue
        if line.startswith('#'):
            continue
        if '=' not in line:
            continue

        k, v = line.split('=', 1)
        k = k.strip()
        v = v.strip()
        if len(k) == 0 or len(v) == 0:
            continue
        VERSION_DATA[k] = v

if 'PKG_MAJOR' not in VERSION_DATA or \
        'PKG_MINOR' not in VERSION_DATA or \
        'PKG_EXTRA' not in VERSION_DATA:
    raise Exception("Malformed pkgconfig.mk")

if 'PKG_TAG' in VERSION_DATA:
    VERSION = '{PKG_MAJOR}.{PKG_MINOR}.{PKG_EXTRA}.{PKG_TAG}'.format(**VERSION_DATA)
else:
    VERSION = '{PKG_MAJOR}.{PKG_MINOR}.{PKG_EXTRA}'.format(**VERSION_DATA)

if SYSTEM == 'darwin':
    LIBRARY_FILE = "libunicorn.dylib"
    MAC_LIBRARY_FILE = "libunicorn*.dylib"
    STATIC_LIBRARY_FILE = None
elif SYSTEM in ('win32', 'cygwin'):
    LIBRARY_FILE = "unicorn.dll"
    STATIC_LIBRARY_FILE = "unicorn.lib"
else:
    LIBRARY_FILE = "libunicorn.so"
    STATIC_LIBRARY_FILE = None

def clean_bins():
    shutil.rmtree(LIBS_DIR, ignore_errors=True)
    shutil.rmtree(HEADERS_DIR, ignore_errors=True)

def copy_sources():
    """Copy the C sources into the source directory.
    This rearranges the source files under the python distribution
    directory.
    """
    src = []

    os.system('make -C %s clean' % os.path.join(ROOT_DIR, '../..'))
    shutil.rmtree(SRC_DIR, ignore_errors=True)
    os.mkdir(SRC_DIR)

    shutil.copytree(os.path.join(ROOT_DIR, '../../qemu'), os.path.join(SRC_DIR, 'qemu/'))
    shutil.copytree(os.path.join(ROOT_DIR, '../../msvc'), os.path.join(SRC_DIR, 'msvc/'))
    shutil.copytree(os.path.join(ROOT_DIR, '../../include'), os.path.join(SRC_DIR, 'include/'))
    # make -> configure -> clean -> clean tests fails unless tests is present
    shutil.copytree(os.path.join(ROOT_DIR, '../../tests'), os.path.join(SRC_DIR, 'tests/'))
    try:
        # remove site-specific configuration file
        # might not exist
        os.remove(os.path.join(SRC_DIR, 'qemu/config-host.mak'))
    except OSError:
        pass

    src.extend(glob.glob(os.path.join(ROOT_DIR, "../../*.[ch]")))
    src.extend(glob.glob(os.path.join(ROOT_DIR, "../../*.mk")))

    src.extend(glob.glob(os.path.join(ROOT_DIR, "../../Makefile")))
    src.extend(glob.glob(os.path.join(ROOT_DIR, "../../LICENSE*")))
    src.extend(glob.glob(os.path.join(ROOT_DIR, "../../README.md")))
    src.extend(glob.glob(os.path.join(ROOT_DIR, "../../*.TXT")))
    src.extend(glob.glob(os.path.join(ROOT_DIR, "../../RELEASE_NOTES")))
    src.extend(glob.glob(os.path.join(ROOT_DIR, "../../make.sh")))
    src.extend(glob.glob(os.path.join(ROOT_DIR, "../../CMakeLists.txt")))
    src.extend(glob.glob(os.path.join(ROOT_DIR, "../../pkgconfig.mk")))

    for filename in src:
        outpath = os.path.join(SRC_DIR, os.path.basename(filename))
        log.info("%s -> %s" % (filename, outpath))
        shutil.copy(filename, outpath)

def build_libraries():
    """
    Prepare the unicorn directory for a binary distribution or installation.
    Builds shared libraries and copies header files.

    Will use a src/ dir if one exists in the current directory, otherwise assumes it's in the repo
    """
    cwd = os.getcwd()
    clean_bins()
    os.mkdir(HEADERS_DIR)
    os.mkdir(LIBS_DIR)

    # copy public headers
    shutil.copytree(os.path.join(BUILD_DIR, 'include', 'unicorn'), os.path.join(HEADERS_DIR, 'unicorn'))

    # check if a prebuilt library exists
    # if so, use it instead of building
    if os.path.exists(os.path.join(ROOT_DIR, 'prebuilt', LIBRARY_FILE)):
        shutil.copy(os.path.join(ROOT_DIR, 'prebuilt', LIBRARY_FILE), LIBS_DIR)
        if STATIC_LIBRARY_FILE is not None and os.path.exists(os.path.join(ROOT_DIR, 'prebuilt', STATIC_LIBRARY_FILE)):
            shutil.copy(os.path.join(ROOT_DIR, 'prebuilt', STATIC_LIBRARY_FILE), LIBS_DIR)
        return

    # otherwise, build!!
    os.chdir(BUILD_DIR)

    try:
        subprocess.check_call(['msbuild', '/help'])
    except:
        has_msbuild = False
    else:
        has_msbuild = True

    if has_msbuild and SYSTEM == 'win32':
        plat = 'Win32' if platform.architecture()[0] == '32bit' else 'x64'
        conf = 'Debug' if os.getenv('DEBUG', '') else 'Release'
        subprocess.call(['msbuild', '-m', '-p:Platform=' + plat, '-p:Configuration=' + conf], cwd=os.path.join(BUILD_DIR, 'msvc'))

        obj_dir = os.path.join(BUILD_DIR, 'msvc', plat, conf)
        shutil.copy(os.path.join(obj_dir, LIBRARY_FILE), LIBS_DIR)
        shutil.copy(os.path.join(obj_dir, STATIC_LIBRARY_FILE), LIBS_DIR)
    else:
        # platform description refs at https://docs.python.org/2/library/sys.html#sys.platform
        new_env = dict(os.environ)
        new_env['UNICORN_BUILD_CORE_ONLY'] = 'yes'
        cmd = ['sh', './make.sh']
        if SYSTEM == "cygwin":
            if IS_64BITS:
                cmd.append('cygwin-mingw64')
            else:
                cmd.append('cygwin-mingw32')
        elif SYSTEM == "win32":
            if IS_64BITS:
                cmd.append('cross-win64')
            else:
                cmd.append('cross-win32')

        subprocess.call(cmd, env=new_env)

        if SYSTEM == 'darwin':
            for file in glob.glob(MAC_LIBRARY_FILE):
                try:
                    shutil.copy(file, LIBS_DIR, follow_symlinks=False)
                except:
                    shutil.copy(file, LIBS_DIR)
        else:
            shutil.copy(LIBRARY_FILE, LIBS_DIR)
        try:
            # static library may fail to build on windows if user doesn't have visual studio installed. this is fine.
            if STATIC_LIBRARY_FILE is not None:
                shutil.copy(STATIC_LIBRARY_FILE, LIBS_DIR)
        except:
            print('Warning: Could not build static library file! This build is not appropriate for a binary distribution')
            # enforce this
            if 'upload' in sys.argv:
                sys.exit(1)
    os.chdir(cwd)


class custom_sdist(sdist):
    def run(self):
        clean_bins()
        copy_sources()
        return sdist.run(self)

class custom_build(build):
    def run(self):
        if 'LIBUNICORN_PATH' in os.environ:
            log.info("Skipping building C extensions since LIBUNICORN_PATH is set")
        else:
            log.info("Building C extensions")
            build_libraries()
        return build.run(self)

class custom_bdist_egg(bdist_egg):
    def run(self):
        self.run_command('build')
        return bdist_egg.run(self)

def dummy_src():
    return []

cmdclass = {}
cmdclass['build'] = custom_build
cmdclass['sdist'] = custom_sdist
cmdclass['bdist_egg'] = custom_bdist_egg

if 'bdist_wheel' in sys.argv and '--plat-name' not in sys.argv:
    idx = sys.argv.index('bdist_wheel') + 1
    sys.argv.insert(idx, '--plat-name')
    name = get_platform()
    if 'linux' in name:
        # linux_* platform tags are disallowed because the python ecosystem is fubar
        # linux builds should be built in the centos 5 vm for maximum compatibility
        # see https://github.com/pypa/manylinux
        # see also https://github.com/angr/angr-dev/blob/master/bdist.sh
        sys.argv.insert(idx + 1, 'manylinux1_' + platform.machine())
    elif 'mingw' in name:
        if IS_64BITS:
            sys.argv.insert(idx + 1, 'win_amd64')
        else:
            sys.argv.insert(idx + 1, 'win32')
    else:
        # https://www.python.org/dev/peps/pep-0425/
        sys.argv.insert(idx + 1, name.replace('.', '_').replace('-', '_'))

try:
    from setuptools.command.develop import develop
    class custom_develop(develop):
        def run(self):
            log.info("Building C extensions")
            build_libraries()
            return develop.run(self)

    cmdclass['develop'] = custom_develop
except ImportError:
    print("Proper 'develop' support unavailable.")

def join_all(src, files):
    return tuple(os.path.join(src, f) for f in files)

long_desc = '''
Unicorn is a lightweight, multi-platform, multi-architecture CPU emulator framework
based on [QEMU](http://qemu.org).

Unicorn offers some unparalleled features:

- Multi-architecture: ARM, ARM64 (ARMv8), M68K, MIPS, SPARC, and X86 (16, 32, 64-bit)
- Clean/simple/lightweight/intuitive architecture-neutral API
- Implemented in pure C language, with bindings for Crystal, Clojure, Visual Basic, Perl, Rust, Ruby, Python, Java, .NET, Go, Delphi/Free Pascal, Haskell, Pharo, and Lua.
- Native support for Windows & *nix (with Mac OSX, Linux, *BSD & Solaris confirmed)
- High performance via Just-In-Time compilation
- Support for fine-grained instrumentation at various levels
- Thread-safety by design
- Distributed under free software license GPLv2

Further information is available at http://www.unicorn-engine.org
'''

import codecs
import os
import sys

HERE = os.path.abspath(os.path.dirname(__file__))
LIBS_DIR = os.path.join(ROOT_DIR, "zebracorn", "lib")
HEADERS_DIR = os.path.join(ROOT_DIR, "zebracorn", "include")

def read(*parts):
    with codecs.open(os.path.join(HERE, *parts), "rb", "utf-8") as f:
        return f.read()

LONG = read(os.path.join("..", os.path.join("..", "README.md")))
VERSION="0.0.0"

setup(
    provides=["zebracorn"],
    packages=["zebracorn"],
    name="zebracorn",
    version=VERSION,
    author="Zeropoint Dynamics",
    author_email="zelos@zeropointdynamics.com",
    description="A Unicorn CPU emulator fork customized for the zelos binary emulator.",
    long_description=LONG,
    long_description_content_type="text/markdown",
    url="https://github.com/zeropointdynamics/zebracorn",
    classifiers=[
        "Topic :: Security",
        "Topic :: System :: Emulators",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Developers",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: Linux",
        "Natural Language :: English",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    requires=["ctypes"],
    cmdclass=cmdclass,
    zip_safe=True,
    include_package_data=True,
    is_pure=True,
    package_data={
        "zebracorn": ["lib/*", "include/unicorn/*"]
    }
)
sys.exit(0)
setup(


    provides=['unicorn'],
    packages=['unicorn'],
    name='unicorn',
    version=VERSION,
    author='Nguyen Anh Quynh',
    author_email='aquynh@gmail.com',
    description='Unicorn CPU emulator engine',
    long_description=long_desc,
    long_description_content_type="text/markdown",
    url='http://www.unicorn-engine.org',
    classifiers=[
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ],
    requires=['ctypes'],
    cmdclass=cmdclass,
    zip_safe=True,
    include_package_data=True,
    is_pure=True,
    package_data={
        'unicorn': ['lib/*', 'include/unicorn/*']
    }
)



#!/usr/bin/env python

import zebracorn
from zebracorn import *

import regress

class MmapSeg(regress.RegressTest):

    def test_seg1(self):
        u = zebracorn.Uc(zebracorn.UC_ARCH_X86, zebracorn.UC_MODE_32)
        u.mem_map(0x2000, 0x1000)
        u.mem_read(0x2000, 1)

        for i in range(50):
            u = zebracorn.Uc(zebracorn.UC_ARCH_X86, zebracorn.UC_MODE_32)
            u.mem_map(i*0x1000, 0x1000)
            u.mem_read(i*0x1000, 1)

        for i in range(20):
            with self.assertRaises(UcError):
                u = zebracorn.Uc(zebracorn.UC_ARCH_X86, zebracorn.UC_MODE_32)
                u.mem_map(i*0x1000, 5)
                u.mem_read(i*0x1000, 1)

    def test_seg2(self):
        uc = Uc(UC_ARCH_X86, UC_MODE_32)
        uc.mem_map(0x0000, 0x2000)
        uc.mem_map(0x2000, 0x4000)
        uc.mem_write(0x1000, 0x1004 * ' ')
        self.assertTrue(1,
            'If not reached, then we have BUG (crash on x86_64 Linux).')

if __name__ == '__main__':
    regress.main()



#!/usr/bin/python
# By Mariano Graziano

from zebracorn import *
from zebracorn.x86_const import *

import regress, struct

mu = 0

class Init(regress.RegressTest):

    def init_zebracorn(self, ip, sp, counter):
        global mu
        #print "[+] Emulating IP: %x SP: %x - Counter: %x" % (ip, sp, counter)
        mu = Uc(UC_ARCH_X86, UC_MODE_64)
        mu.mem_map(0x1000000, 2 * 1024 * 1024)
        mu.mem_write(0x1000000, "\x90")
        mu.mem_map(0x8000000, 8 * 1024 * 1024)
        mu.reg_write(UC_X86_REG_RSP, sp)
        content = self.generate_value(counter)
        mu.mem_write(sp, content)
        self.set_hooks()
   
    def generate_value(self, counter):
        start = 0xffff880026f02000
        offset = counter * 8
        address = start + offset
        return struct.pack("<Q", address)
 
    def set_hooks(self):
        global mu
        mu.hook_add(UC_HOOK_MEM_READ_UNMAPPED | UC_HOOK_MEM_WRITE_UNMAPPED, self.hook_mem_invalid)
        mu.hook_add(UC_HOOK_MEM_FETCH_UNMAPPED, self.hook_mem_fetch_unmapped)
    
    def hook_mem_invalid(self, uc, access, address, size, value, user_data):
        global mu
        print "[ HOOK_MEM_INVALID - Address: %s ]" % hex(address)
        if access == UC_MEM_WRITE_UNMAPPED:
            print ">>> Missing memory is being WRITE at 0x%x, data size = %u, data value = 0x%x" %(address, size, value)
            address_page = address & 0xFFFFFFFFFFFFF000
            mu.mem_map(address_page, 2 * 1024 * 1024)
            mu.mem_write(address, str(value))
            return True
        else:
            return False

    def hook_mem_fetch_unmapped(self, uc, access, address, size, value, user_data):
        global mu
        print "[ HOOK_MEM_FETCH - Address: %s ]" % hex(address).strip("L")
        print "[ mem_fetch_unmapped: faulting address at %s ]" % hex(address).strip("L")
        mu.mem_write(0x1000003, "\x90") 
        mu.reg_write(UC_X86_REG_RIP, 0x1000001)
        return True

    def runTest(self):
        global mu
        ips = list(xrange(0x1000000, 0x1001000, 0x1))
        sps = list(xrange(0x8000000, 0x8001000, 0x1))
        j = 0
        for i in ips:
            j += 1
            index = ips.index(i)
            self.init_zebracorn(i, sps[index], j)
            mu.emu_start(0x1000000, 0x1000000 + 0x1)

if __name__ == '__main__':
    regress.main()
